;;; base-extensions.el --- Extenciones/paquetes instalados y su configuración

;; Author: kj <webmaster@outcontrol.net>
;; URL: https://git.kj2.me/kj/confi-emacs-actual

;;; Commentary:

;;; Code:

;; Iconos principalmente para ser usados por neotree
(use-package all-the-icons
  :defer t
  :straight t)

;; Iconos en Ivy (allthe icons)
(use-package all-the-icons-ivy-rich
  :defer t
  :straight t
  :ensure t
  :init
  (all-the-icons-ivy-rich-mode 1)
  :config
  (setq all-the-icons-ivy-rich-color-icon t))

;; Reemplazar y buscar mejorado
(use-package anzu
  :ensure t
  :straight t
  :config
  (global-anzu-mode +1)
  (global-set-key [remap query-replace] 'anzu-query-replace)
  (global-set-key [remap query-replace-regexp] 'anzu-query-replace-regexp))

;; Automcompletado
(use-package company
  :defer t
  :straight t
  :init
  (global-company-mode)
  :config
  (setq company-dabbrev-downcase nil)    ; autocompletado case-sensitive.
  (setq company-idle-delay 0)            ; mostrar autocompletado lo más rápido posible
  (setq company-minimum-prefix-length 1) ; mostrar autocompletado desde que se coloca la primera letra.
  (setq company-backends '((company-capf :with company-yasnippet)
                           (company-files :with company-yasnippet)
                           (company-dabbrev-code company-ctags company-keywords :with company-yasnippet)
                           (company-dabbrev :with company-yasnippet)))
  )

;; Hacer que el autocompletado se vea más bonito con íconos
(use-package company-box
  :straight t
  :defer t
  :init
  (company-box-mode 1))

;; Usar autocompletado con ctags y company
(use-package company-ctags :defer t :straight t)

;; Poner la info acerca del autocompletado del autocompletado mas rápido
(use-package company-quickhelp
  :defer t
  :straight t
  :ensure t
  :custom
  (company-quickhelp-delay 2)
  (company-quickhelp-mode))

;; Autocompletado para shell scripting.
(use-package company-shell
  :defer t
  :straight t
  :config
  (add-to-list 'company-backends '(company-shell company-shell-env company-fish-shell))
  )

;; Autocompletado para el minibuffer (counsel e ivy)
(use-package counsel :straight t :defer t)

;; Autocompletado de proyectos en counsel (projectile)
(use-package counsel-projectile
  :defer t
  :straight t
  :config
  (counsel-projectile-mode))

;; Un bonito y sencillo panel de inicio
(use-package dashboard
  :defer t
  :straight t
  :init
  (dashboard-setup-startup-hook)
  :config
  (setq dashboard-set-file-icons t)
  (setq dashboard-set-heading-icons t)
  (setq dashboard-set-file-icons t)
  (setq dashboard-startup-banner
        (expand-file-name "duck-small.png" user-emacs-directory))
  (setq dashboard-items '(
                          (agenda . 10)
                          ;;(recents . 10)
                          ))
  )

;; Mover líneas o regiones
(use-package drag-stuff
  :defer t
  :straight t
  :init
  (drag-stuff-global-mode 1)
  :config
  (drag-stuff-define-keys))

;; La línea bonita esa de abajo
(use-package doom-modeline
  :defer t
  :straight t
  :ensure t
  :init (doom-modeline-mode 1)
  :config
  (setq doom-modeline-project-detection 'auto
        doom-modeline-buffer-file-name-style 'relative-from-project
        doom-modeline-major-mode-color-icon t
        doom-modeline-buffer-modification-icon t
        doom-modeline-indent-info nil
        doom-modeline-persp-name t))

;; Emmet - Una ayuda para escribir HTML rápidamente (escribre doc, luego preciona C-j y lo entenderás)
(use-package emmet-mode
  :defer t
  :straight t
  :hook (html-mode . emmet-mode))

;; Revisar sintaxis en vivo
(use-package flycheck
  ;; :diminish flycheck-mode
  :defer t
  :straight t
  :hook (after-init . global-flycheck-mode) ; Habilitar flycheck en todos los modos
  )

;; Mostrar los errores de flycheck en un pop up
(use-package flycheck-popup-tip
  :defer t
  :straight t
  :hook (flycheck-mode . flycheck-popup-tip-mode))

;; Git Gutter - Marca a la izq. si una linea ha sido agregada, editada o eliminada desde el último commit.
(use-package git-gutter
  :defer t
  :straight t
  :init
  (global-git-gutter-mode +1))

;; Highlight en los números.
(use-package highlight-numbers
  :defer t
  :straight t
  :ensure t
  :hook
  (prog-mode . highlight-numbers-mode))

;; Mostrar info del panel inferior de otra manera
(use-package ivy
  :defer t
  :straight t
  :config
  (ivy-mode 1)
  (setq ivy-use-virtual-buffers nil)
  )

(use-package ivy-rich
  :defer t
  :straight t
  :ensure t
  :config
  (ivy-rich-mode 1))

;; Languaje server protocol
(use-package lsp-mode
  :defer t
  :straight t
  :custom
  (lsp-headerline-breadcrumb-enable nil)
  :config
  (setq lsp-completion-provider :none))

;; Interface para lsp
(use-package lsp-ui
  :defer t
  :straight t)

;; Magia para git
(use-package magit
  :defer t
  :straight t)

;; Modo markdown
(use-package markdown-mode
  :defer t
  :straight t)

;; Multiple vterm
(use-package multi-vterm :ensure t :defer t :straight t)

;; Code Folding
(use-package origami
  :defer t
  :straight t
  :init
  (global-origami-mode))

;; Paquete para manejo de proyectos
(use-package projectile
  :defer t
  :straight t
  :diminish projectile-mode
  :custom ((projectile-completion-system 'ivy))
  :init
  (projectile-mode)
  :config
  ;; Rutas de archivos temporales.
  (setq projectile-cache-file (expand-file-name "projectile.cache" temp-dir))
  (setq projectile-known-projects-file (expand-file-name
                                        "projectile-bookmarks.eld" temp-dir))
  ;; Carpetas donde tienes tus proyectos (deben tener un archivo .projectile o un repro git iniciado).
  (when (file-directory-p "~/Proyectos")
    (setq projectile-project-search-path '("~/Proyectos")))
  (when (file-directory-p "~/mnt/Nginx")
    (setq projectile-project-search-path (append
                                          projectile-project-search-path
                                          '("~/mnt/Nginx"))))
  (setq projectile-switch-project-action #'projectile-dired) ; Usar dired cuando se elija un proyecto.

  ;; Cambiar el título de la ventana de emacs
  (setq frame-title-format
        '(
          (:eval
           (let ((project-name (projectile-project-name)))
             (unless (string= "-" project-name)
               (format "[%s] " project-name))))
          "%b"
          " - Emacs")
        )
   )

;; Recentf - Guarda registro de los archivos abiertos recientemente
(use-package recentf
  :defer t
  :straight t
  :config
  (setq recentf-save-file
        (recentf-expand-file-name (concat temp-dir "/recentf")))
  (recentf-mode 1))

;; Guardar la posición del cursor en un archivo para volver allí cuando se lo vuelva a abrir.
(use-package saveplace
  :defer t
  :straight t
  :init
  (save-place-mode 1)
  :config
  (setq save-place-file (locate-user-emacs-file (concat temp-dir "/places"))))

;; Autocompletado de parentesis, corchetes, llaves, etc.
(use-package smartparens
  :defer t
  :straight t
  :init
  (smartparens-global-mode t))

;; Mejorando el scroll
(use-package smooth-scrolling
  :defer t
  :straight t
  :init
  (smooth-scrolling-mode 1)
  :config
  (setq mouse-wheel-scroll-amount
        '(8 ((shift) . 1) ((control) . nil)))    ; Cambia el scroll a 8 líneas a la vez, 1 cuando se preciona SHIFT y saltos de página cuando presionas CTRL
  (setq mouse-wheel-progressive-speed nil)     ; Deshabilita la velocidad progresiva del scroll (mientras más scroll haces, mas rápido va)
  )

;; use-package - No
(use-package use-package :defer t :straight t)

;; Terminal
(use-package vterm :ensure t :defer t :straight t)

;; Cuando iniicias un atajo de teclas te muestra las posibilidades
(use-package which-key
  :defer t
  :straight t
  :init
  (which-key-mode))

;; Restaurar el estado de los frames
(use-package winner
  :defer t
  :straight t
  :init
  (winner-mode 1))

;; Permitir snippets
(use-package yasnippet
  :defer t
  :straight t
  :ensure t
  :custom
  (yas-prompt-functions '(yas-completing-prompt))
  :hook
  ((prog-mode feature-mode)  . yas-minor-mode-on)
  (html-mode . yas-minor-mode))

;; Coleción de snippets
(use-package yasnippet-snippets
  :ensure t
  :defer t
  :straight t)

;; Org-Mode
(require 'org-config)

(provide 'base-extensions)
;;; base-extensions.el ends here
